// Oleg Petruny proprietary.

#pragma once

#include "GameFramework/Character.h"

#include "PlayerBase.generated.h"

enum class EActivatorType : uint8;

DECLARE_DYNAMIC_MULTICAST_DELEGATE(FPlayerMovedDelegate);

USTRUCT(BlueprintType)
struct FPlayerLock
{
    GENERATED_BODY()

    uint8 walk : 1;
    uint8 jump : 1;
    uint8 run : 1;
    uint8 interaction : 1;
    uint8 camera : 1;
    uint8 inventory : 1;
};

UCLASS(Blueprintable, BlueprintType)
class APlayerBase : public ACharacter
{
    GENERATED_BODY()

public:
    APlayerBase();

    virtual void Tick(float DeltaTime) override;
    virtual void SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent) override;

    FVector GetCameraLocation();
    FVector GetCameraDirection();
    bool IsMoving() { return isMoving; }
    void LockPlayer(const FPlayerLock lock);
    void UnlockPlayer(const FPlayerLock lock);

    UFUNCTION(BlueprintCallable, Category = Character)
    void TakeItemToLeftHand(class AActor* actor);
    UFUNCTION(BlueprintCallable, Category = Character)
    void TakeItemToRightHand(class AActor* actor);
    UFUNCTION(BlueprintCallable, Category = Character)
    bool IsInInventory(const class AActor* actor);

    UPROPERTY(BlueprintAssignable)
    FPlayerMovedDelegate OnPlayerMoved;
    FVector moveVector;

    class UEnhancedInputComponent* inputComponent = nullptr;

    class AActor* leftPocketItem = nullptr;
    class AActor* rightPocketItem = nullptr;

protected:
    virtual void BeginPlay() override;

    UFUNCTION(BlueprintCallable, Category = CameraMode)
    void SwitchToCameraPawn();

    UFUNCTION(BlueprintCallable, Category = Character)
    void MoveCamera(FVector2D value);
    UFUNCTION(BlueprintCallable, Category = Character)
    void MoveCharacter(FVector2D value);
    virtual void Jump() override;
    UFUNCTION(BlueprintCallable, Category = Character)
    void SwitchRun(bool run);

    UFUNCTION(BlueprintCallable, Category = Character)
    void UpdatePitch(float min, float max);

    UFUNCTION(BlueprintCallable, Category = Character)
    void DropLeftHandItem();
    UFUNCTION(BlueprintCallable, Category = Character)
    void DropRightHandItem();
    UFUNCTION(BlueprintCallable, Category = Character)
    void ShowInventory();

    class APlayerController* playerController;
    UPROPERTY(EditAnywhere)
    float moveSpeed = 200;
    UPROPERTY(EditAnywhere)
    float runSpeedMultiplier = 4;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 walkLocked = 0;
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 jumpLocked = 0;
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 runLocked = 0;
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 interactionLocked = 0;
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 cameraLocked = 0;
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    uint8 inventoryLocked = 0;

    APlayerCameraManager* cameraManager;
    class UCameraComponent* camera;
    UPROPERTY(EditDefaultsOnly, BlueprintReadOnly)
    float minPitch = -80;
    UPROPERTY(EditDefaultsOnly, BlueprintReadOnly)
    float maxPitch = 65;

    UPROPERTY(BlueprintReadOnly)
    bool isMoving = false;

    class AInteractable* lastInteractable = nullptr;
    class USpringArmComponent* cameraSpringArm;

private:
    void LoadInteractablesActivators();
    void InteractableActivated(class AInteractable* interactable, EActivatorType type);
    void InteractableDeactivated(class AInteractable* interactable, EActivatorType type);

    void TakeToInventory(class AActor* actor);
    void DropItem(class AActor* actor);

    FVector oldLocation;
    FRotator rotationInput;

    bool itemDropLocked = false;
    FTimerHandle itemDropLockTimer;
};
