// Oleg Petruny proprietary.

#pragma once

#include "UObject/Object.h"

#include "WidgetsManager.generated.h"

enum class EInputAnimatedWidgetAnimation : uint8;

UCLASS(Blueprintable)
class UWidgetsManager : public UObject
{
    GENERATED_BODY()

public:
    void Init();

    void ShowWidgets();
    void HideWidgets();
    void UpdateWidgetsOwner();

    void ShowInteractionHints(const class UInteractableModificator* modificator);
    void HideInteractionHints(const class UInteractableModificator* modificator);
    void AnimateInteractionHint(const class UInteractableModificator* modificator, const int32 index, const EInputAnimatedWidgetAnimation animation);

    void EnableCutsceneWidget(FText keyText, class FSkipCutsceneDelegate& skipCutsceneDelegate);
    void DisableCutsceneWidget();
    void AnimateCutsceneWidget(const EInputAnimatedWidgetAnimation animation);

    void ShowInventory();
    void ShowInventory(FText key1, FText key2);
    void SetInventoryFirstItem(FText item);
    void SetInventorySecondItem(FText item);

protected:
    UPROPERTY(EditDefaultsOnly)
    TSet<TSubclassOf<class UUserWidget>> permaOverlayWidgets; // never hidden
    TArray<class UUserWidget*> permaOverlayWidgetsInstances;

    UPROPERTY(EditDefaultsOnly)
    TSet<TSubclassOf<class UUserWidget>> overlayWidgets; // hidden in pause
    TArray<class UUserWidget*> overlayWidgetsInstances;

    UPROPERTY(EditDefaultsOnly)
    TSubclassOf<class UInteractableHintWidgetManager> interactableHintWidgetManagerClass;
    class UInteractableHintWidgetManager* interactableHintWidgetManager = nullptr;

    UPROPERTY(EditDefaultsOnly)
    TSubclassOf<class UCutsceneSkipWidget> cutsceneSkipWidgetClass;
    class UCutsceneSkipWidget* cutsceneSkipWidget = nullptr;

    UPROPERTY(EditDefaultsOnly)
    TSubclassOf<class UInventoryWidget> inventoryWidgetClass;
    class UInventoryWidget* inventoryWidget = nullptr;
};
